# encoding=utf8


class Constants(object):
    SCHEDULE_RECORD = 'schedule_record'
    TABLE_RECORD = 'table_record'
    INTERNAL_SUMMARY_OF_SCHEDULE_NUMBER = 'internal_summary_of_schedule_number'
    EXTERNAL_SUMMARY = 'external_summary'
    INTERNAL_SUMMARY_OF_TABLE_NUMBER = 'internal_summary_of_table_number'
    MANUAL_NOTE_RECORD = 'manual_note_record'

    UNKNOWN = 'unknown'

    SINGLE_NUMBER = 'single_number'
    NUMBER_SPAN = 'number_span'
    SUMMARY_NUMBER_SPAN = 'summary_number_span'

    COMPLEX_SEE_REFERENCE = 'nce'
    COMPLEX_SEE_ALSO_REFERENCE = 'nsa'
    DEFINITION = 'ndf'
    SCOPE_NOTE = 'scope_note'
    APPLICATION_INSTRUCTION_NOTE = 'application_instruction_note'
    HISTORY_NOTE = 'history_note'
    AUXILIARY_INSTRUCTION_NOTE = 'auxiliary_instruction_note'
